package com.bull.cimero.pluginEditor.editors.partsOutline;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Collections;
import java.util.List;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.editparts.AbstractTreeEditPart;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import cimeroEditor.CimeroEditorPlugin;
import com.bull.cimero.pluginEditor.editors.figure.GeneriqueCimeroFigure;
import com.bull.cimero.pluginEditor.editors.figure.BCFigure.BCConcreteFigure;
import com.bull.cimero.pluginEditor.editors.figure.SEFigure.SEConcreteFigure;
import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.editors.model.BCModel.BindingComponentModel;
import com.bull.cimero.pluginEditor.editors.model.SEModel.ServiceEngineModel;
import com.bull.cimero.pluginEditor.editors.policies.NodeRemoveEditPolicy;
import com.bull.cimero.pluginEditor.utils.classLoader.CimeroClassLoader;
import com.bull.cimero.pluginEditor.utils.log.CimeroLog;
import com.bull.cimero.pluginEditor.utils.ressources.Ressources;

/**
 * class used to make communication for node model and outline view.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class NodeTreeEditPart extends AbstractTreeEditPart
							implements PropertyChangeListener {

	/**Current factory package.*/
	private static final String CURRENTPACKAGE =
		"com.bull.cimero.pluginEditor.editors.figure";
	/**BC figure package.*/
	private static final String BCPACKAGE = "BCFigure";
	/**SE figure package.*/
	private static final String SEPACKAGE = "SEFigure";
	/**The suffixe figure.*/
	private static final String SUFFIXEFIGURE = "Figure";

	/**
	 * the nodeTreeEditPart constructor.
	 * @param model : the model.
	 */
   public NodeTreeEditPart(final Object model) {
      super(model);
   }

   /**
    * activate method.
    */
   public final void activate() {
      super.activate();
      ((GeneriqueCimeroModel) getModel()).addPropertyChangeListener(this);
   }

   /**
    * desactivate method.
    */
   public final void deactivate() {
      ((GeneriqueCimeroModel) getModel()).removePropertyChangeListener(this);
      super.deactivate();
   }

   /**
    * get the model children.
    * @return the list of the childre of this model.
    */
   protected final List getModelChildren() {
      return Collections.EMPTY_LIST;
   }

   /**
    * event : change a property.
    * @param change : the change event.
    */
   public final void propertyChange(final PropertyChangeEvent change) {
      refreshVisuals();
   }

   /**
    * refresh visuals method.
    */
   protected final void refreshVisuals() {
	   GeneriqueCimeroModel model = (GeneriqueCimeroModel) getModel();
	   setWidgetText(((GeneriqueCimeroModel) getModel()).getName());
	   //set the image
	   setWidgetImage(
			CimeroEditorPlugin.getDefault()
					.getImageDescriptor(getPathToIcon(model)).createImage());
   }

   /**
    * create edit policies method.
    */
   protected final void createEditPolicies() {
	      installEditPolicy(EditPolicy.COMPONENT_ROLE,
	    		  new NodeRemoveEditPolicy());
   }

   /**
    * method used to get the path to the icon associated
    * with a given model.
    * @param model : model
    * @return String : path to the icon
    */
   public final String getPathToIcon(final GeneriqueCimeroModel model) {
	
	    GeneriqueCimeroFigure genFigure = null;
		if (model.isCustomComponent()){
			String customerComponentId = model.getCustomComponentId();
			Ressources ressource = new Ressources();
			String fullPath = CimeroEditorPlugin.getRessourcePath("/ressources/CustomComponents.xml");
			if (fullPath != null) {
				
				if (fullPath.substring(0, "file:".length()).compareTo("file:") == 0) {
					fullPath = fullPath.substring("file:".length());
				}
			
				ressource.setDocument(fullPath);
				
				
				NodeList nodeList = ressource.getXPathNodeSet("/COMPONENTS/COMPONENT"); 
				Node node = null;
				NamedNodeMap attributes = null;
			
				String idTemp = null;
				for (int i = 0; i < nodeList.getLength(); i++) {
					node = nodeList.item(i);				
					attributes = node.getAttributes();
					
					 idTemp = attributes.getNamedItem("id").getNodeValue();
					 String editorImage =attributes.getNamedItem("editorImage").getNodeValue();
					 String errorImage = attributes.getNamedItem("errorImage").getNodeValue();
					 String paletteImage = attributes.getNamedItem("paletteImage").getNodeValue();
					 if (idTemp.equalsIgnoreCase(customerComponentId)){
						  if (model instanceof ServiceEngineModel) {		
							  genFigure =  new SEConcreteFigure(editorImage, errorImage, paletteImage);
						}else{
							genFigure =  new BCConcreteFigure(idTemp, editorImage, errorImage, paletteImage);
						}
					 }
				}
			}
		}else{
	   
			String fullPackageClass = CURRENTPACKAGE;
			String simpleName = model.getClass().getSimpleName();
			//test BC or SE
			if (model instanceof BindingComponentModel) {
				//BC component.
				fullPackageClass += "." + BCPACKAGE;
			} else {
				//SE Component.
				fullPackageClass += "." + SEPACKAGE;
			}
			fullPackageClass += "." + simpleName + SUFFIXEFIGURE;
			//create the figure
	 
			if (model instanceof BindingComponentModel) {
				try {
					genFigure = (GeneriqueCimeroFigure) CimeroClassLoader
							.newInstance(fullPackageClass, new Object[] {});
				} catch (Exception e) {
					CimeroLog.errorLog("the BC figure"
						+ fullPackageClass + "cannot be create", e);
					return "";
				}
			} else {
				try {
					genFigure = (GeneriqueCimeroFigure) CimeroClassLoader
							.newInstance(fullPackageClass, new Object[] {});
				} catch (Exception e) {
					CimeroLog.errorLog("the SE figure"
						+ fullPackageClass + "cannot be create", e);
				return "";
				}
			}
		}
		return genFigure.getPathToIcon();
   }
}
