package com.bull.cimero.pluginEditor.utils.listener;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.IResourceDeltaVisitor;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.IWorkbenchPartSite;
import org.eclipse.ui.part.FileEditorInput;
import com.bull.cimero.pluginEditor.editors.DiagramEditor;
import com.bull.cimero.pluginEditor.utils.log.CimeroLog;

/**
 * class used to listen a file and open or close editor.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello ( andrea.zoppello@eng.it )
 */
public class ResourceTracker implements IResourceChangeListener,
										IResourceDeltaVisitor {

	/**
	 * the diagram editor which call this listener.
	 */
	private DiagramEditor diagramEditor = null;

	/**
	 * default constructor.
	 * @param myDiag : the diagram editor that create this listener
	 */
	public ResourceTracker(final DiagramEditor myDiag) {
		super();
		this.diagramEditor = myDiag;
	}

	/**
	 * the changed event resource function.
	 * @param event : the event.
	 */
	public final void resourceChanged(final IResourceChangeEvent event) {
		IResourceDelta delta = event.getDelta();
		try {
			if (delta != null) {
				delta.accept(this);
			}
		} catch (CoreException exception) {
			CimeroLog.warningLog("", exception);
		}
	}

	/**
	 * @return the editor
	 *
	 */
	private IFileEditorInput getEditorInput() {
		return (IFileEditorInput) diagramEditor.getEditorInput();
	}

	/**
	 * @return the IworkbenchPartSite using by the diagram editor.
	 */
	private IWorkbenchPartSite getSite() {
		return diagramEditor.getSite();
	}


	/**
	 * @see org.eclipse.core.resources.IResourceDeltaVisitor#visit(
	 * org.eclipse.core.resources.IResourceDelta)
	 * @param delta : the Iresource delta.
	 * @return true if the action is ok.
	 */
	public final boolean visit(final IResourceDelta delta) {
		if (delta == null || !delta.getResource().equals(
				((IFileEditorInput) getEditorInput()).getFile())) {
				return true;
		}

		if (delta.getKind() == IResourceDelta.REMOVED) {
			Display display = getSite().getShell().getDisplay();
			if ((IResourceDelta.MOVED_TO & delta.getFlags()) == 0) { // if
				// the file was deleted
				// NOTE: The case where an open, unsaved file is deleted is
				// being handled by the
				// PartListener added to the Workbench in the initialize()
				// method.
				display.asyncExec(new Runnable() {
					public void run() {
						if (!diagramEditor.isDirty()) {
							diagramEditor.closeEditor(false);
						}
					}
				});
			} else {
				// else if it was moved or renamed
				final IFile newFile = ResourcesPlugin.getWorkspace()
				.getRoot().getFile(delta.getMovedToPath());
				display.asyncExec(new Runnable() {
					public void run() {
						diagramEditor.setInput(new FileEditorInput(newFile));
					}
				});
			}
		} else if (delta.getKind() == IResourceDelta.CHANGED) {
			if (!diagramEditor.isDirty()) {
				//if the file is changed in the workspace but not with editor.
				final IFile newFile = ResourcesPlugin.getWorkspace()
				.getRoot().getFile(delta.getFullPath());
				Display display = getSite().getShell().getDisplay();
				display.asyncExec(new Runnable() {

					public void run() {
						diagramEditor.setInput(new FileEditorInput(newFile));
					}
				});
			}
		}
		return false;
	}
}

